"""Default configuration file contents"""

# Remember to add tests for keys into test_influxlogger.py
DEFAULT_CONFIG_STR = """
[zmq]
pub_sockets = ["ipc:///tmp/influxlogger_pub.sock", "tcp://*:58770"]

[db]
host = "127.0.0.1"
# enable for real use
#port = 8486
#ssl = true
#verify_ssl = true
username = "user"
password = "password"
db = "mydb"

[[subscriptions]]  # make this kind of section for each subscription
zmq = ["ipc:///tmp/something.sock", "topic"]
measurement = "mymeas"
tag_fields = ["field1"]  # which fields to use as tags
exclude_fields = ["field2"]  # optional, not used if include_fields is used
include_fields = ["field3", "field4"]  # optional if not set all fields are included

""".lstrip()



"""Main classes for dsinfluxlogger"""
from __future__ import annotations
from typing import cast, Optional, List, Any, Dict
from dataclasses import dataclass, field
import logging

from aioinflux import InfluxDBClient  # type: ignore
from flatten_dict import flatten  # type: ignore
from datastreamcorelib.pubsub import PubSubMessage, Subscription
from datastreamcorelib.datamessage import PubSubDataMessage
from datastreamservicelib.service import SimpleService

LOGGER = logging.getLogger(__name__)


@dataclass
class InfluxLoggerService(SimpleService):
    """Main class for dsinfluxlogger"""

    _client: Optional[InfluxDBClient] = field(init=False, default=None)

    async def teardown(self) -> None:
        """teardown"""
        if self._client:
            await self._client.close()
        await super().teardown()

    def reload(self) -> None:
        """Load configs, restart sockets"""
        super().reload()
        self.create_task(self.reload_async())

    async def reload_async(self) -> None:
        """async reload"""
        if self._client:
            await self._client.close()
        self._client = InfluxDBClient(**self.config["db"])
        # die early if db cannot be contacted
        await self._client.ping()

        for subinfo in self.config["subscriptions"]:
            sub = Subscription(
                subinfo["zmq"][0],
                subinfo["zmq"][1],
                self.message_callback,
                decoder_class=PubSubDataMessage,
                metadata={"config": subinfo},
            )
            self.psmgr.subscribe_async(sub)

    async def write_point(self, point: Dict[str, Any]) -> None:
        """Wrap the write to a task"""
        if self._client is None:
            raise RuntimeError("we have no client")
        LOGGER.debug("About to write {}".format(repr(point)))
        await self._client.write(point)

    async def message_callback(self, sub: Subscription, msg: PubSubMessage) -> None:
        """Callback for the example subscription"""
        msg = cast(PubSubDataMessage, msg)
        config = sub.metadata["config"]
        tag_fields: List[str] = config.get("tag_fields", [])
        exclude_fields: List[str] = config.get("exclude_fields", [])
        include_fields: Optional[List[str]] = config.get("include_fields", None)
        # start creating the point
        # FIXME: use dataframes for batched writes
        #  see https://aioinflux.readthedocs.io/en/stable/usage.html#writing-dictionary-like-objects
        point = {"time": msg.data["systemtime"], "measurement": config["measurement"], "tags": {}, "fields": {}}
        # resolve tags
        for name in tag_fields:
            point["tags"][name] = msg.data[name]
        # resolve fields to include
        if include_fields is None:
            include_fields = []
            for name in msg.data.keys():
                if name in exclude_fields + ["systemtime"]:
                    continue
                include_fields.append(name)
        # Start adding values
        values_dict: Dict[str, Any] = {}
        for name in include_fields:
            values_dict[name] = msg.data[name]
        # Flatten the values
        point["fields"] = flatten(values_dict, reducer="underscore")
        self.create_task(self.write_point(point))



# -*- coding: utf-8 -*-
from setuptools import setup

package_dir = \
{'': 'src'}

packages = \
['dsinfluxlogger']

package_data = \
{'': ['*']}

install_requires = \
['aioinflux>=0.9,<0.10',
 'datastreamservicelib>=1.10,<2.0',
 'flatten-dict>=0.3,<0.4']

entry_points = \
{'console_scripts': ['dsinfluxlogger = '
                     'dsinfluxlogger.console:influxlogger_cli']}

setup_kwargs = {
    'name': 'dsinfluxlogger',
    'version': '0.3.1',
    'description': 'Log datamessages to influxdb',
    'long_description': '==============\ndsinfluxlogger\n==============\n\nLog PubSubDataMessages (see datastreamcorelib) to InfluxDB, this is a pretty quick and dirty implementation\nand definitely not optimal.\n\nFor optimal write performance we should add a plugin system that allows\none to convert the payload from the PubSubDataMessages into line-protocol decorated\ncustom classes https://aioinflux.readthedocs.io/en/stable/usage.html#writing-user-defined-class-objects\n\nThe quick-and-dirty optimization is to batch writes into pandas dataframes, which has the not\ninsignificant drawback of adding pandas/numpy to our requirements.\n\nDocker\n------\n\nFor more controlled deployments and to get rid of "works on my computer" -syndrome, we always\nmake sure our software works under docker.\n\nIt\'s also a quick way to get started with a standard development environment.\n\nSSH agent forwarding\n^^^^^^^^^^^^^^^^^^^^\n\nWe need buildkit_::\n\n    export DOCKER_BUILDKIT=1\n\n.. _buildkit: https://docs.docker.com/develop/develop-images/build_enhancements/\n\nAnd also the exact way for forwarding agent to running instance is different on OSX::\n\n    export DOCKER_SSHAGENT="-v /run/host-services/ssh-auth.sock:/run/host-services/ssh-auth.sock -e SSH_AUTH_SOCK=/run/host-services/ssh-auth.sock"\n\nand Linux::\n\n    export DOCKER_SSHAGENT="-v $SSH_AUTH_SOCK:$SSH_AUTH_SOCK -e SSH_AUTH_SOCK"\n\nCreating a development container\n^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^\n\nBuild image, create container and start it::\n\n    docker build --ssh default --target devel_shell -t dsinfluxlogger:devel_shell .\n    docker create --name dsinfluxlogger_devel -p 58770:58770 -v `pwd`":/app" -it -v /tmp:/tmp `echo $DOCKER_SSHAGENT` dsinfluxlogger:devel_shell\n    docker start -i dsinfluxlogger_devel\n\npre-commit considerations\n^^^^^^^^^^^^^^^^^^^^^^^^^\n\nIf working in Docker instead of native env you need to run the pre-commit checks in docker too::\n\n    docker exec -i dsinfluxlogger_devel /bin/bash -c "pre-commit install"\n    docker exec -i dsinfluxlogger_devel /bin/bash -c "pre-commit run --all-files"\n\nYou need to have the container running, see above. Or alternatively use the docker run syntax but using\nthe running container is faster::\n\n    docker run -it --rm -v `pwd`":/app" dsinfluxlogger:devel_shell -c "pre-commit run --all-files"\n\nTest suite\n^^^^^^^^^^\n\nYou can use the devel shell to run py.test when doing development, for CI use\nthe "tox" target in the Dockerfile::\n\n    docker build --ssh default --target tox -t dsinfluxlogger:tox .\n    docker run -it --rm -v `pwd`":/app" `echo $DOCKER_SSHAGENT` dsinfluxlogger:tox\n\nProduction docker\n^^^^^^^^^^^^^^^^^\n\nThere\'s a "production" target as well for running the application (change "myconfig.toml" for config file)::\n\n    docker build --ssh default --target production -t dsinfluxlogger:latest .\n    docker run -it --name dsinfluxlogger -v myconfig.toml:/app/config.toml -p 58770:58770 -it -v /tmp:/tmp `echo $DOCKER_SSHAGENT` dsinfluxlogger:latest\n\n\nLocal Development\n-----------------\n\nTLDR:\n\n- Create and activate a Python 3.8 virtualenv (assuming virtualenvwrapper)::\n\n    mkvirtualenv -p `which python3.8` my_virtualenv\n\n- change to a branch::\n\n    git checkout -b my_branch\n\n- install Poetry: https://python-poetry.org/docs/#installation\n- Install project deps and pre-commit hooks::\n\n    poetry install\n    pre-commit install\n    pre-commit run --all-files\n\n- Ready to go, try the following::\n\n    dsinfluxlogger --defaultconfig >config.toml\n    dsinfluxlogger -vv config.toml\n\nRemember to activate your virtualenv whenever working on the repo, this is needed\nbecause pylint and mypy pre-commit hooks use the "system" python for now (because reasons).\n\nRunning "pre-commit run --all-files" and "py.test -v" regularly during development and\nespecially before committing will save you some headache.\n',
    'author': 'Eero af Heurlin',
    'author_email': 'eero.afheurlin@iki.fi',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://gitlab.com/advian-oss/python-dsinfluxlogger/',
    'package_dir': package_dir,
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



"""CLI entrypoints for dsinfluxlogger"""
from typing import Any
import asyncio
import sys
import logging
from pathlib import Path

import click

from datastreamcorelib.logging import init_logging
from dsinfluxlogger.defaultconfig import DEFAULT_CONFIG_STR
from dsinfluxlogger import __version__
from dsinfluxlogger.service import InfluxLoggerService


LOGGER = logging.getLogger(__name__)


def dump_default_config(ctx: Any, param: Any, value: bool) -> None:  # pylint: disable=W0613
    """Print the default config and exit"""
    if not value:
        return
    click.echo(DEFAULT_CONFIG_STR)
    if ctx:
        ctx.exit()


@click.command()
@click.version_option(version=__version__)
@click.option("-l", "--loglevel", help="Python log level, 10=DEBUG, 20=INFO, 30=WARNING, 40=CRITICAL", default=30)
@click.option("-v", "--verbose", count=True, help="Shorthand for info/debug loglevel (-v/-vv)")
@click.option(
    "--defaultconfig",
    is_flag=True,
    callback=dump_default_config,
    expose_value=False,
    is_eager=True,
    help="Show default config",
)
@click.argument("configfile", type=click.Path(exists=True))
def influxlogger_cli(configfile: Path, loglevel: int, verbose: int) -> None:
    """Log datamessages to influxdb"""
    if verbose == 1:
        loglevel = 20
    if verbose >= 2:
        loglevel = 10
    init_logging(loglevel)
    LOGGER.setLevel(loglevel)

    service_instance = InfluxLoggerService(Path(configfile))

    exitcode = asyncio.get_event_loop().run_until_complete(service_instance.run())
    sys.exit(exitcode)
